from apiaudio.helper_classes import UploadableResource


class Media(UploadableResource):
    OBJECT_NAME = "media"
    resource_path = "/file"
    audio_resource_path = "/file/customaudio"

    @classmethod
    def get_download_url(cls, mediaId: str = ""):
        if not mediaId:
            raise ValueError("please specify a mediaId")

        url = (
            cls.list(mediaId=mediaId, downloadUrl=True)
            .get("savedAudioFiles", [{}])[0]
            .get("downloadUrl", None)
        )

        if not url:
            raise ValueError("url could not be retrieved")

        return url

    @classmethod
    def list(cls, tags: str = "", mediaId: str = "", downloadUrl: bool = False) -> dict:
        request_params = {}

        if downloadUrl:
            request_params["target"] = "download"

        if mediaId:
            return cls._get_request(
                path_param=cls.audio_resource_path,
                request_params={**request_params, **{"mediaId": mediaId}},
            )
        if tags:
            return cls._get_request(
                path_param=cls.audio_resource_path,
                request_params={**request_params, **{"tags": tags}},
            )
        return cls._get_request(
            path_param=cls.audio_resource_path, request_params=request_params
        )

    @classmethod
    def list_tags(cls):
        return cls._get_request(path_param=f"{cls.audio_resource_path}/tags")

    @classmethod
    def download(cls, mediaId: str = "", destination: str = "."):
        url = cls.get_download_url(mediaId=mediaId)

        return cls._download_request(url=url, destination=destination)



from apiaudio.helper_classes import (
    CreatableResource,
    ListableResource,
    RetrievableResource,
)


class Script(ListableResource, CreatableResource, RetrievableResource):
    OBJECT_NAME = "script"
    resource_path = "/script"
    random_url = "/script/random"

    @classmethod
    def get_random_text(cls, category=None):
        return cls._get_request(
            path_param=cls.random_url, request_params={"category": category}
        )



from apiaudio.api_resources.script import Script
from apiaudio.api_resources.speech import Speech
from apiaudio.api_resources.voice import Voice
from apiaudio.api_resources.mastering import Mastering
from apiaudio.api_resources.sound import Sound
from apiaudio.api_resources.syncTTS import SyncTTS
from apiaudio.api_resources.media import Media



import apiaudio
import requests
import shutil
import os
import requests
from requests.exceptions import HTTPError


class APIRequest:
    def _api_key_checker(api_key=None):
        api_key_error_message = "Please specify a valid api_key or create one here:\n https://console.api.audio"
        PLACEHOLDERS = ["your-key", "APIKEY", "API_KEY"]
        if api_key == None or api_key in PLACEHOLDERS:
            # aflr_key is for backward compatibility with the old name of env vars.
            api_key = os.environ.get("apiaudio_key", os.environ.get("aflr_key"))

            if api_key == None:
                raise TypeError(f"No api_key has been found. {api_key_error_message}")

        if not isinstance(api_key, str):
            raise TypeError("api_key must be of type string.")

        if len(api_key) < 32:
            raise ValueError(
                f"api_key has less than 32 characters. {api_key_error_message}"
            )
        apiaudio.api_key = api_key
        return

    @classmethod
    def _build_header(cls):
        cls._api_key_checker(apiaudio.api_key)
        return {"x-api-key": apiaudio.api_key}

    @classmethod
    def _post_request(cls, json, url=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"
        
        headers = cls._build_header()
        r = requests.post(url=url, headers=headers, json=json)

        # speech timeouts
        if r.status_code == 504:
            r = requests.get(url=url, headers=headers, params=json)

        cls._expanded_raise_for_status(r)

        return r.json()

    @classmethod
    def _put_request(cls, data, url=None, headers=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"

        r = requests.put(url=url, headers=headers, data=data)

        cls._expanded_raise_for_status(r)
        
        if r.status_code != 200:
            raise ValueError("Error performing the PUT request")

        # since aws s3 does not return a body on PUT requests, 
        # r.json() does not work here
        return r

    @classmethod
    def _get_request(cls, url=None, path_param=None, request_params=None):
        url = url or f"{apiaudio.api_base}{cls.resource_path}"

        headers = cls._build_header()

        if path_param:
            url = f"{apiaudio.api_base}{path_param}"

        if request_params:
            r = requests.get(url=url, headers=headers, params=request_params)
        else:
            r = requests.get(url=url, headers=headers)

        cls._expanded_raise_for_status(r)

        return r.json()

    @classmethod
    def _download_request(cls, url, destination):
        if type(url) is not str:
            raise TypeError("Error retrieving the audio files.")

        local_filename = f"{destination}/{url.split('/')[-1].split('?')[0]}"
        local_filename = local_filename.replace("%243ct10n", "section")

        with requests.get(url, stream=True) as r:
            cls._expanded_raise_for_status(r)

            with open(local_filename, "wb") as f:
                shutil.copyfileobj(r.raw, f)

        return local_filename

    @classmethod
    def config_test(cls):
        return f"Configured to transact {cls.OBJECT_NAME} objects to {apiaudio.api_base}{cls.resource_path} with api_key = {apiaudio.api_key}"

    @classmethod
    def _expanded_raise_for_status(self, res):
        """
        Take a "requests" response object and expand the raise_for_status method to return errors from API
        @param res:
        @return: None
        """
        try:
            res.raise_for_status()
        except HTTPError as e:
            if res.json():
                raise HTTPError(
                    "{} \n Error Message from API: \n {}".format(res.json(), str(e))
                )
            else:
                raise e
        return




import aflr
from aflr import Script, Sound, Speech, Mastering, Voice
import pprint
import pytest
import os 

aflr.api_key = os.getenv("AFLR_API_KEY_PEADAR")
def test_english_voices():
    print("List test:\nAll voices:")
    print(len(Voice().list().get("voices")))
    print("\nFiltered voices:")
    assert len(Voice().list(language="english").get("voices")) == 16

def test_spanish_voices():
    assert len(Voice.list(language="spanish").get("voices")) == 7

def test_list_parameters():
    assert len(Voice.list_parameters()) > 0 

def test_sound_design():
    sounds = Sound.list_sound_templates()
    city_nights=sounds.get("templateFiles").get("citynights")
    # Still not sure how to get this to work properly
    assert len(city_nights) > 0
    assert list(city_nights[1]) == ['citynights__music.wav']

def test_create_scripts():
    create_script= Script.create(scriptText="<<sectionName::question>> Hey! Do you know we support multiple voices from different providers in the same script? I am a polly voice from Amazon. <<sectionName::answer>> I am Azure voice from Microsoft. I think Azure voices sound awesome.")
    scriptId = create_script.get("scriptId")
    create_speech = Speech.create(scriptId=scriptId)
    create_tts = {'message': 'Success. Text-to-speech is in progress.'}
    assert create_speech == create_tts

def test_create_mastering():
    create_script= Script().create(scriptText="<<sectionName::question>> Hey! Do you know we support multiple voices from different providers in the same script? I am a polly voice from Amazon. <<sectionName::answer>> I am Azure voice from Microsoft. I think Azure voices sound awesome.")
    scriptId = create_script.get("scriptId")
    Speech().create(scriptId=scriptId)
    mastering = Mastering.create(scriptId=scriptId)
    create_mastering = {'Message': 'Mastering request was successful'}
    assert len(mastering) > 0
    assert mastering == create_mastering



# Aflorithmic Python bindings
# API docs at https://docs.api.audio/
# Authors:
# Antonio Tripiana <antonio@aflorithmic.ai>

# Configuration variables

api_key = None
client_id = None
api_base = "https://v1.api.audio"
upload_api_base = "https://file.api.audio"  # not implemented yet.

# future
api_version = None
verify_ssl_certs = True
proxy = None
default_http_client = None
app_info = None
enable_telemetry = True
max_network_retries = 0

# Set to either 'debug' or 'info', controls console logging
log = None

# API resources
from apiaudio.api_resources import *



from apiaudio.helper_classes import (
    CreatableResource,
    DownloadableResource,
    RetrievableResource,
)


class Speech(CreatableResource, RetrievableResource, DownloadableResource):
    OBJECT_NAME = "speech"
    resource_path = "/speech"



from apiaudio.helper_classes import ListableResource


class Voice(ListableResource):
    OBJECT_NAME = "voice"
    resource_path = "/voice"
    list_parameters_path = "/voice/parameter"

    @classmethod
    def list_parameters(cls):
        return cls._get_request(path_param=cls.list_parameters_path)



import os

from .api_request import APIRequest


class ListableResource(APIRequest):
    @classmethod
    def list(cls, **args):
        if args:
            return cls._get_request(request_params=args)
        else:
            return cls._get_request()


class CreatableResource(APIRequest):
    @classmethod
    def create(cls, **params):
        return cls._post_request(json=params)


class RetrievableResource(APIRequest):
    @classmethod
    def retrieve(cls, scriptId, section=None, parameters=None, public=None, vast=None, endFormat=None):
        params = parameters.copy() if parameters else {}
        params.update({"scriptId": scriptId})

        if section:
            params.update({"section": section})

        if public is not None:
            params.update({"public": public})

        if vast is not None:
            params.update({"vast": vast})
        
        if endFormat is not None:
            params.update({"endFormat": endFormat})

        if cls.OBJECT_NAME == "script":
            return cls._get_request(path_param=f"{cls.resource_path}/{scriptId}")

        if hasattr(cls, "file_url"):
            return cls._get_request(path_param=f"{cls.file_url}", request_params=params)
        else:
            return cls._get_request(request_params=params)


class DownloadableResource(APIRequest):
    @classmethod
    def download(
        cls,
        scriptId,
        section=None,
        parameters=None,
        public=None,
        vast=None,
        destination=".",
    ):
        parameters = parameters or {}

        try:
            audio_files = cls.retrieve(scriptId, section, parameters, public, vast)
            audio_files.keys()
        except Exception:
            raise TypeError(
                "Error retrieving the audio files. Make sure you can retrieve them with the same parameters and try again."
            )

        if "url" in audio_files.keys():
            local_filename = cls._download_request(
                url=audio_files.get("url"), destination=destination
            )
            return local_filename

        local_filenames = []
        for key, value in audio_files.items():
            # Review "value"! list of string...
            local_filename = cls._download_request(url=value, destination=destination)
            local_filenames.append(local_filename)

        return local_filenames


class UploadableResource(APIRequest):
    @classmethod
    def upload(cls, file_path: str = "", tags: str = ""):
        payload = open(file_path, "rb")
        filename = os.path.basename(file_path)
        headers = {"Content-Type": "audio/mpeg"}

        request_params = {"filename": filename}

        if tags:
            request_params["tags"] = tags

        # get presigned URL
        url = cls._get_request(
            path_param=f"{cls.audio_resource_path}/uploadurl",
            request_params=request_params,
        )

        mediaId = url["mediaId"]
        fileUploadUrl = url["fileUploadUrl"]

        response = cls._put_request(url=fileUploadUrl, headers=headers, data=payload)

        payload.close()

        response = {
            "message": f"Success. Use mediaId to retrieve this file",
            "mediaId": mediaId,
        }

        return response



from apiaudio.helper_classes import (
    CreatableResource,
    DownloadableResource,
    ListableResource,
    RetrievableResource,
)


class Sound(
    CreatableResource, DownloadableResource, RetrievableResource, ListableResource
):
    OBJECT_NAME = "sound"
    resource_path = "/sound"
    list_path = "/sound/template"
    file_url = "/file/sound"
    list_parameters_path = "/sound/parameter"

    @classmethod
    def list_parameters(cls):
        return cls._get_request(path_param=cls.list_parameters_path)

    @classmethod
    def list(cls, **args):
        return cls._get_request(path_param=cls.list_path, request_params=args)
